import logging

from safehouse import config
from .event_manager import EventManager


logger = logging.Logger(__name__)


def init(
    *,
    origin: str,
) -> EventManager:
    project = config.project
    if not project:
        raise Exception("no project defined")
    return EventManager(origin, project)



import json
import logging
import requests
from typing import List, Optional

from safehouse.types import JsonBlob


logger = logging.Logger(__name__)
EVENTS_URL = "http://localhost:9000"


class ClientError(Exception):
    def __init__(self, detail):
        super().__init__(detail)


def headers():
    return {
        "Accept": "application/json",
        "Content-Type": "application/json",
    }


def post(
        *,
        data: Optional[JsonBlob] = None,
        endpoint: str,
        headers: Optional[JsonBlob] = headers(),
        query_params: Optional[JsonBlob] = None,
        acceptable_status_codes: Optional[List[int]] = [200],
) -> requests.Response:
        response = requests.post(
            EVENTS_URL + endpoint,
            headers=headers,
            data=data,
        )
        validate_response(response, acceptable_status_codes=acceptable_status_codes)
        return response


def validate_response(
        response: requests.Response,
        acceptable_status_codes: Optional[List[int]] = [200],
):
    if response.status_code not in acceptable_status_codes:
        error_string = str(response)
        error_detail = error_string
        try:
            error_string = response.json()
            if type(error_string) == list:
                error_detail = ', '.join(error_string)
            else:
                error_detail = error_string['detail']
        except json.JSONDecodeError:
            pass
        raise ClientError(error_detail)



import logging

from .event import Event, EventType
from safehouse import config
from safehouse.services.client import events as events_service


logger = logging.Logger(__name__)


class EventManager:
    def __init__(self, origin: str, project: str):
        self.origin = origin
        self.project = project
        self.load_events()

    def __getattr__(self, name):
        event_type = self.event_types.get(name)
        if event_type:
            return Event(event_type=event_type, origin=self.origin)
        raise AttributeError(f"'{type(self)}' object has no attribute '{name}'")

    def load_events(self):
        event_types_list = events_service.register_event_types(config.project.events)
        self.event_types = {
            e['name']: EventType(e['name'], e['version'], e['attributes'])
            for e in event_types_list
        }
        logger.info(f"registered_event_types=={self.event_types}")



import json
from pathlib import Path
from typing import Optional

from safehouse.services import local
from safehouse.types import JsonBlob
from . import exceptions


MODES = (
    'live',
    'local',
    'standalone',
)


class Project:
    def __init__(
        self,
        *,
        events: Optional[JsonBlob]={},
        filepath: Path, 
        mode: Optional[str]='local',
        name: str,
        org_name: str,
    ):
        self.events = events
        self.filepath = filepath
        self.mode = mode.lower()
        self.name = name.lower()
        self.org_name = org_name.lower()
        self.validate()

    def __str__(self) -> str:
        return f"{self.org_name}.{self.name} ({self.filepath})"

    @property
    def json(self) -> JsonBlob:
        return {
            'events': self.events,
            'mode': self.mode,
            'name': self.name,
            'organization': self.org_name,
        }

    @property
    def is_local(self) -> bool:
        return self.mode == 'local'

    @property
    def json_file(self) -> Path:
        return self.filepath / 'project.json'

    def save(self):
        with open(self.json_file, 'w') as f:
            json.dump(self.json, f, indent=4)

    def validate(self):
        assert self.filepath is not None
        assert self.name is not None
        assert self.org_name is not None
        if self.mode not in MODES:
            raise Exception(f"unsupported mode: '{self.mode}'")
        if self.is_local and not local.is_running():
            raise exceptions.ProjectError(f"{self} needs safehouse services to be running locally")


def from_safehouse_dir(safehouse_dir: Path) -> Project:
    project_json_file = str(safehouse_dir / "project.json")
    with open(project_json_file, 'r', encoding='utf-8') as f:
        project_dict: JsonBlob = json.load(f)
        events = project_dict.get('events', {})
        mode = project_dict.get('mode', 'local')
        name = project_dict.get('name')
        org_name = project_dict.get('organization')
        return Project(
            events=events,
            filepath=safehouse_dir,
            mode=mode,
            name=name,
            org_name=org_name,
        )



#!/usr/bin/env python
import logging
import sys
from typing import Dict

from safehouse.apps.console import commands
from safehouse.apps.console.commands.types import Runnable


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


supported_commands: Dict[str, Runnable] = {
    'orgs': commands.orgs,
    'projects': commands.projects,
    'services': commands.services,
    'users': commands.users,
}

def run():
    args = [arg.lower() for arg in sys.argv[1:]]
    if len(args) > 0:
        command_name = args[0]
        command = supported_commands.get(command_name)
        if not command:
            logger.error(f"'safehouse {command_name}' not supported")
            logger.info(usage())
            sys.exit(-1)
        try:
            command.run(args[1:])
        except commands.exceptions.CommandError as e:
            logger.error(e)
            sys.exit(-1)
        except Exception as e:
            logger.exception(e)
            sys.exit(-1)
    else:
        print(usage())
        sys.exit(0)


def usage() -> str:
    return f"usage: safehouse <{'|'.join(supported_commands.keys())}>"

if __name__ == "__main__":
    run()



from typing import List

from ..exceptions import CommandError
from . import init


def run(args: List[str]):
    if len(args) != 2:
        raise CommandError(usage())
    command = args[0]
    if command == 'init':
        init.run(args[1:])
    else:
        raise CommandError(f"unsupported projects command '{command}'")


def usage() -> str:
    return "safehouse projects init <organization.project>"
 


from safehouse import projects, utils

project = None
safehouse_dir = None

def init():
    global project, safehouse_dir
    safehouse_dir = utils.find_directory_in_parent_dirs('.safehouse')
    project = projects.from_safehouse_dir(safehouse_dir) if safehouse_dir else None

init()
